// BtClient.java
// Data exchange with BtServer.java

import java.io.*;
import javax.microedition.lcdui.*;
import ch.aplu.gidlet.*;
import ch.aplu.bluetooth.*;

public class BtClient extends Gidlet
{
  // ------------- Inner class TransferThread -----------
  private class TransferThread extends Thread
  {
    public void run()
    {
      try
      {
        for (int n = 1; n <= 100; n++)
        {
          dos.writeInt(n);
          dos.flush();

          // Blocking, throws IOException when server closes
          // connection or BluetoothClient.disconnect() is called
          int k = dis.readInt();
          it.setText("Sent: " + n + "\nReply: " + k);
        }
      }
      catch (IOException ex)
      {
        it.setText("readInt() unblocked");
        delay(2000);
      }
      bc.disconnect();
      it.setText("Transfer thread terminated");
    }
  }
  // ------------- End of inner class -------------------

  private final boolean isVerbose = false;

  private String[] names = {"", ""};
  private BluetoothClient bc;
  private DataInputStream dis;
  private DataOutputStream dos;
  private TextField tf1;
  private TextField tf2;
  private StringItem it;
  private volatile boolean ok = false;
  private MForm form = new MForm("BtClient");

  public void main()
  {
    if (isVerbose)
      VerboseWriter.init("e:/debug/BtClient.txt");
    // Get old values from StringStore to avoid to much fingertips
    StringStore store = new StringStore("BtClientStore");
    String content = store.read();
    if (content != null)
      names = split(content, ";");

    tf1 = form.addInputField("Server name:", names[0]);
    tf2 = form.addInputField("Service name:", names[1]);
    it = form.addOutputField();
    form.show();
    while (!ok)
      waitOk();

    store.write(names[0] + ";" + names[1]);
    it.setText("Trying to connect to\n" + names[0] +
                "\nwith service\n" + names[1]);

    bc = new BluetoothClient(names[0], names[1]);
    if (isVerbose)
      bc.setVerbose(true);
    if (!bc.connect())
    {
      it.setText("Connection failed");
      VerboseWriter.close();
      return;
    }

    dis = new DataInputStream(bc.getInputStream());
    dos = new DataOutputStream(bc.getOutputStream());

    it.setText("Connection established");
    delay(2000);
    new TransferThread().start();
  }

  public void doOk()
  {
    names[0] = form.getText(tf1);
    names[1] = form.getText(tf2);
    if (names[0].equals("") || names[1].equals(""))
      it.setText("Illegal entry");
    else
      ok = true;
  }

  public void doExit()
  {
    if (bc != null)
      bc.disconnect();
    if (isVerbose)
      VerboseWriter.close();
    super.doExit();
  }

}
