// BtPeerInqGidlet.java
// Inquiry Machine, use BtPeerAns as 'Answer Machine' on a PC

import ch.aplu.bluetooth.*;
import ch.aplu.gidlet.*;
import javax.microedition.lcdui.*;

public class BtPeerInqGidlet extends Gidlet implements BtPeerListener
{
  private BluetoothPeer bp = null;
  private String btName = "";
  private final String serviceName = "WordInvert";
  private TextField tf;
  private StringItem it;
  private MForm form;
  private volatile boolean isConnected = false;
  private volatile boolean ok = false;

  public void main()
  {
    // Read from StringStore to avoid fingertips
    StringStore store = new StringStore("InqStore");
    String content = store.read();
    if (content != null)
      btName = content;
    form = new MForm("Inquiry machine");
    tf = form.addInputField("Enter Bluetooth name:", btName);
    it = form.addOutputField();
    form.show();
    while (!ok)
      waitOk();  // Wait for Bluetooth name entry

    store.write(btName);
    it.setText("Trying to connect to\n" + btName +
                "\nwith service\n" + serviceName);

    bp = new BluetoothPeer(btName, serviceName, this, false);
    if (!bp.isConnected())  // We are a server, wait for a connection
      putSleep();
    form = new MForm("String Inverser");
    tf = form.addInputField("Enter a string:", "");
    it = form.addOutputField();
    form.addOkButton();
    form.show();
  }

  // Callback when data are received
  public void receiveDataBlock(int[] data)
  {
    String s = "";
    int size = data.length;
    for (int i = 0; i < size; i++)
      s = s + (char)data[i];
    it.setText("Reply: " + s);
    wakeUp();
  }

  // Callback when connection is established or lost
  public void notifyConnection(boolean connected)
  {
    if (connected)
    {
      isConnected = true;
      it.setText("Connection established");
      delay(2000);
      wakeUp();
    }
    else
    {
      isConnected = false;
      it.setText("Connection lost");
      form.removeOkButton();
      bp.releaseConnection();
    }
  }

  // Callback when OK soft button is hit
  public void doOk()
  {
    if (!isConnected)  // First form
    {
      btName = form.getText(tf);
      if (btName.equals(""))
        it.setText("Illegal entry");
      else
        ok = true;
    }
    else // Second form
    {
      String value = form.getText(tf);
      int size = value.length();
      int[] data = new int[size];
      for (int i = 0; i < size; i++)
        data[i] = value.charAt(i);
      bp.sendDataBlock(data);
      putSleep(); // Wait for reply
    }
  }

  // Callback when the Exit soft button is hit
  public void doExit()
  {
    if (bp != null)
      bp.releaseConnection();
    super.doExit();
  }
}


