// PlzGidlet.java
// chplz must reside in res subdirectory
// Text file, each line has ZIP code in format: nnnn;location<cr><lf> 

import ch.aplu.gidlet.*;
import javax.microedition.lcdui.*;

public class PlzGidlet extends Gidlet
{

  private final String version = "V1.0";

// =============== Inner class SelectForm ========================
private class SelectForm extends MForm implements ItemStateListener
{
  private StringItem si;
  private ChoiceGroup cg =
    new ChoiceGroup("Ort whlen", Choice.POPUP);

  private SelectForm(String title)
  {
    super(title);
    setItemStateListener(this);
    append(cg);
    si = addOutputField();
    si.setText("Mit ENTER Ort whlen.");
  }

  private void fill(int startIndex, int endIndex)
  {
    cg.deleteAll();
    for (int i = startIndex; i <= endIndex; i++)
      cg.append(ortWithAccents[i], null);
 }

  public void itemStateChanged(Item item)
  {
    if (item == cg)
    {
      mainForm.displayResultWithOffset(cg.getSelectedIndex());
      mainForm.show(false);
    }
  }
}

// =================== Inner class MainForm ==================
private class MainForm extends MForm implements ItemStateListener
{
  // ------------- Inner class MyCommandListener -------------
  private class MyCommandListener implements CommandListener
  {
    public void commandAction(Command c, Displayable d)
    {
      if (c == exitCommand)
        notifyDestroyed();

      if (c == searchCommand)
      {
        tf2.setString("");
        String searchKey = removeAccents(getText(tf1));
        if (searchKey.length() < 2)
        {
          si.setText("Ort zu kurz.\n\n" + searchText);
          return;
        }
        startIndex = findFirst(ort, searchKey);
        if (startIndex > -1)
        {
          int endIndex = findLast(ort, searchKey);
          nbFound = endIndex - startIndex + 1;
          selectForm.fill(startIndex, endIndex);
        }
        else
          nbFound = 0;

        switch (nbFound)
        {
          case 0:
            si.setText("Kein Ort gefunden.\n\n" + searchText);
            break;

          case 1:
            displayResult(startIndex);
            break;

          default:
            selectForm.show(false);
            break;
        }
      }

      if (c == nextCommand)
      {
        tf1.setString("");
        tf2.setString("");
        removeCommand(nextCommand);
        addCommand(searchCommand);
        si.setText(searchText);
        isNext = false;
      }
    }
  }
  // ------------- End of inner class ------------------------

  private TextField tf1;
  private TextField tf2;
  private StringItem si;
  private int startIndex;
  private Command searchCommand = new Command("Search", Command.OK, 1);
  private Command nextCommand = new Command("Next", Command.OK, 1);
  private Command exitCommand = new Command("Exit", Command.EXIT, 1);
  private boolean isNext = false;
  private final String searchText =
    "Bitte mind. 2 erste Buchstaben des Orts eingeben und Search klicken";

  private MainForm(String title)
  {
    super(title);
  }

  private void init()
  {
    tf1 = addInputField("Ort:", "");
    tf2 = addInputField("PLZ:", "");
    si = addOutputField();
    si.setText(nbRecords + " Orte in Datenbank.\n\n" + searchText);
    removeExitButton();
    addCommand(exitCommand);
    addCommand(searchCommand);
    setCommandListener(new MyCommandListener());
    setItemStateListener(this);
  }

  private int findFirst(String[] a, String searchKey)
  // Return index of first element starting with searchKey, -1 if not found
  {
    String key = searchKey.toLowerCase();
    for (int i = 0; i < nbRecords; i++)
    {
      String ort = a[i].toLowerCase();
      if (ort.startsWith(key))
        return i;
    }
    return -1;
  }

  private int findLast(String[] a, String searchKey)
  // Return index of last element starting with searchKey, -1 if not found
  {
    String key = searchKey.toLowerCase();
    for (int i = nbRecords - 1; i >= 0; i--)
    {
      String ort = a[i].toLowerCase();
      if (ort.startsWith(key))
        return i;
    }
    return -1;
  }

  private void displayResultWithOffset(int index)
  {
    displayResult(startIndex + index);
  }

  private void displayResult(int index)
  {
    tf1.setString(ortWithAccents[index]);
    tf2.setString(plz[index]);
    si.setText("Postleitzahl gefunden.\n\nNext klicken oder Ort editieren");
    removeCommand(searchCommand);
    addCommand(nextCommand);
    isNext = true;
  }

  public void itemStateChanged(Item item)
  {
    if (item == tf1 && isNext)
    {
      isNext = false;
      tf2.setString("");
      si.setText(searchText);
      removeCommand(nextCommand);
      addCommand(searchCommand);
    }
  }
}
// ================== End of inner classes ======================


  private final String fileUrl = "/res/chplz";
  private String[] plz;
  private String[] ortWithAccents;
  private String[] ort;
  private int nbRecords;
  private int nbFound;
  private MainForm mainForm;
  private SelectForm selectForm;

  public void main()
  {
    selectForm = new SelectForm("Ort auswhlen");
    mainForm = new MainForm("Postleitzahl " + version);
    mainForm.show();

    mainForm.addOutputField().setText(
      "Postleitzahl von Orten in der Schweiz.\n" +
      "----- www.aplu.ch -----\n\n" +
      "Lade Datenbank. Bitte warten...");

    if (!getRecords())
    {
      mainForm.delete(0);
      mainForm.addOutputField().setText("Postleitzahldatei nicht gefunden");
      return;
    }
    mainForm.delete(0);
    mainForm.init();
  }

  private boolean getRecords()
  {
    String content = readFile(fileUrl);
    if (content == null)
      return false;
    String[] records = split(content, "\r\n");
    nbRecords = records.length - 1;
    ortWithAccents = new String[nbRecords];
    ort = new String[nbRecords];
    plz = new String[nbRecords];

    for (int i = 0; i < nbRecords; i++)
    {
      String[] parts = split(records[i], ";");
      plz[i] = parts[0];
      ortWithAccents[i] = parts[1];
      ort[i] = removeAccents(parts[1]);
    }
    return true;
  }

  private static String removeAccents(String input)
  // Character set is ISO-8859 (8bit)
  {
    StringBuffer output = new StringBuffer();

    for (int i = 0; i < input.length(); i++)
    {
      switch (input.charAt(i))
      {
        case '':
        case '':
        case '':
        case '':
        case '':
        case '':
          output.append("A");
          break;

        case '':
          output.append("AE");
          break;

        case '':
          output.append("C");
          break;

        case '':
        case '':
        case '':
        case '':
          output.append("E");
          break;

        case '':
        case '':
        case '':
        case '':
          output.append("I");
          break;

        case '':
          output.append("D");
          break;

        case '':
          output.append("N");
          break;

        case '':
        case '':
        case '':
        case '':
        case '':
        case '':
          output.append("O");
          break;

        case '\u0152':
          output.append("OE");
          break;

        case '':
          output.append("TH");
          break;

        case '':
        case '':
        case '':
        case '':
          output.append("U");
          break;

        case '':
        case '\u0178':
          output.append("Y");
          break;

        case '':
        case '':
        case '':
        case '':
        case '':
        case '':
          output.append("a");
          break;

        case '':
          output.append("ae");
          break;

        case '':
          output.append("c");
          break;

        case '':
        case '':
        case '':
        case '':
          output.append("e");
          break;

        case '':
        case '':
        case '':
        case '':
          output.append("i");
          break;

        case '':
          output.append("d");
          break;

        case '':
          output.append("n");
          break;

        case '':
        case '':
        case '':
        case '':
        case '':
        case '':
          output.append("o");
          break;

        case '\u0153':
          output.append("oe");
          break;

        case '':
          output.append("ss");
          break;

        case '':
          output.append("th");
          break;

        case '':
        case '':
        case '':
        case '':
          output.append("u");
          break;

        case '':
        case '':
          output.append("y");
          break;

        default:
          output.append(input.charAt(i));
          break;
      }
    }
    return output.toString();
  }
}
