// VocReadGidlet.java

import ch.aplu.gidlet.*;
import javax.microedition.lcdui.*;

public class VocReadGidlet extends Gidlet
{
  // ------------- Inner class MyCommandListener -------------

  private class MyCommandListener implements CommandListener
  {

    public void commandAction(Command c, Displayable d)
    {
      if (c == exitCommand)
        notifyDestroyed();

      if (c == nextCommand)
      {
        curIndex++;
        if (curIndex == nb)
        {
          tf1.setString("");
          tf2.setString("");
          si.setText("Lesson finished");
          f.removeCommand(nextCommand);
          f.removeCommand(showCommand);
          f.addCommand(exitCommand);
        }
        else
        {
          tf1.setString(question[curIndex]);
          tf2.setString("");
          si.setText("Record: " + (curIndex + 1) + " of " + nb);
          f.removeCommand(exitCommand);
          f.addCommand(showCommand);
        }
      }

      if (c == showCommand)
      {
        tf2.setString(answer[curIndex]);
        f.removeCommand(showCommand);
        f.addCommand(exitCommand);
      }
    }
  }
  // ------------- End of inner class ------------------------

  private MForm f;
  private MFile mf = new MFile();
  private TextField tf1;
  private TextField tf2;
  private StringItem si;
  private Command nextCommand = new Command("Next", Command.OK, 1);
  private Command showCommand = new Command("Show", Command.EXIT, 1);
  private Command exitCommand = new Command("Exit", Command.EXIT, 1);
  private String[] question;
  private String[] answer;
  private int nb; // Total nb of records
  private int curIndex; // Current record index

  public void main()
  {
    f = new MForm("VocReadGidlet");
    si = f.addOutputField();
    f.show();
    if (!selectLesson())
    {
      si.setText("Read of vocabulary file failed");
      return;
    }
    getRecords();
    if (nb == 0)
    {
      si.setText("Vocabulary file empty");
      return;
    }
    initForm();
    curIndex = 0;
    si.setText("Record: 1 of " + nb);
    tf1.setString(question[curIndex]);
  }

  private boolean selectLesson()
  // Return true, if successful, otherwise false
  {
    mf.findFirstRoot();
    if (mf.getPath().equals("c:/"))  // No access on c:, try next
      mf.findNextRoot();
    mf.setPath(mf.getPath() + "voc/");
    String dirPath = mf.getPath();
    mf = new MFile(dirPath);
    if (!mf.exists())  // Directory does not exist
      return false;

    mf.setPath(dirPath + "lesson1");
    if (!mf.canRead())  // File unreadable
      return false;
    return true;
  }

  private void initForm()
  {
    f.delete(0);  // Remove string item
    tf1 = f.addInputField("Deutsch:", "");
    tf2 = f.addInputField("English:", "");
    si = f.addOutputField();

    f.removeExitButton();
    f.addCommand(nextCommand);
    f.addCommand(showCommand);
    f.setCommandListener(new MyCommandListener());
  }

  private void getRecords()
  {
    String content = mf.read();

    if (content.equals("")) // Empty file
    {
      nb = 0;
      return;
    }

    String[] records = split(content, "\r\n");
    nb = records.length - 1;
    question = new String[nb];
    answer = new String[nb];

    for (int i = 0; i < nb; i++)
    {
      String[] parts = split(records[i], "|");
      question[i] = parts[0];
      answer[i] = parts[1];
    }
  }
}




